@@ -9,6 +9,9 @@ from typing import (
 
 import neo4j
 from neo4j import GraphDatabase
+from neo4j.api import (
+    SECURITY_TYPE_SECURE, SECURITY_TYPE_SELF_SIGNED_CERTIFICATE, parse_neo4j_uri,
+)
 from pyhocon import ConfigFactory, ConfigTree
 
 from databuilder.extractor.base_extractor import Extractor
@@ -25,15 +28,19 @@ class Neo4jExtractor(Extractor):
     MODEL_CLASS_CONFIG_KEY = 'model_class'
     NEO4J_AUTH_USER = 'neo4j_auth_user'
     NEO4J_AUTH_PW = 'neo4j_auth_pw'
+    # in Neo4j (v4.0+), we can create and use more than one active database at the same time
+    NEO4J_DATABASE_NAME = 'neo4j_database'
     NEO4J_MAX_CONN_LIFE_TIME_SEC = 'neo4j_max_conn_life_time_sec'
     NEO4J_ENCRYPTED = 'neo4j_encrypted'
     """NEO4J_ENCRYPTED is a boolean indicating whether to use SSL/TLS when connecting."""
     NEO4J_VALIDATE_SSL = 'neo4j_validate_ssl'
     """NEO4J_VALIDATE_SSL is a boolean indicating whether to validate the server's SSL/TLS cert against system CAs."""
+    NEO4J_DRIVER = 'neo4j_driver'
 
-    DEFAULT_CONFIG = ConfigFactory.from_dict({NEO4J_MAX_CONN_LIFE_TIME_SEC: 50,
-                                              NEO4J_ENCRYPTED: True,
-                                              NEO4J_VALIDATE_SSL: False})
+    DEFAULT_CONFIG = ConfigFactory.from_dict({
+        NEO4J_MAX_CONN_LIFE_TIME_SEC: 50,
+        NEO4J_DATABASE_NAME: neo4j.DEFAULT_DATABASE
+    })
 
     def init(self, conf: ConfigTree) -> None:
         """
@@ -43,8 +50,36 @@ class Neo4jExtractor(Extractor):
         self.conf = conf.with_fallback(Neo4jExtractor.DEFAULT_CONFIG)
         self.graph_url = conf.get_string(Neo4jExtractor.GRAPH_URL_CONFIG_KEY)
         self.cypher_query = conf.get_string(Neo4jExtractor.CYPHER_QUERY_CONFIG_KEY)
-        self.driver = self._get_driver()
+        self.db_name = self.conf.get_string(Neo4jExtractor.NEO4J_DATABASE_NAME)
+        driver = conf.get(Neo4jExtractor.NEO4J_DRIVER, None)
+        if driver:
+            self.driver = driver
+        else:
+            uri = conf.get_string(Neo4jExtractor.GRAPH_URL_CONFIG_KEY)
+            driver_args = {
+                'uri': uri,
+                'max_connection_lifetime': self.conf.get_int(Neo4jExtractor.NEO4J_MAX_CONN_LIFE_TIME_SEC),
+                'auth': (conf.get_string(Neo4jExtractor.NEO4J_AUTH_USER),
+                         conf.get_string(Neo4jExtractor.NEO4J_AUTH_PW)),
+            }
 
+            # if URI scheme not secure set `trust`` and `encrypted` to default values
+            # https://neo4j.com/docs/api/python-driver/current/api.html#uri
+            _, security_type, _ = parse_neo4j_uri(uri=uri)
+            if security_type not in [SECURITY_TYPE_SELF_SIGNED_CERTIFICATE, SECURITY_TYPE_SECURE]:
+                default_security_conf = {'trust': neo4j.TRUST_ALL_CERTIFICATES, 'encrypted': True}
+                driver_args.update(default_security_conf)
+
+            # if NEO4J_VALIDATE_SSL or NEO4J_ENCRYPTED are set in config pass them to the driver
+            validate_ssl_conf = conf.get(Neo4jExtractor.NEO4J_VALIDATE_SSL, None)
+            encrypted_conf = conf.get(Neo4jExtractor.NEO4J_ENCRYPTED, None)
+            if validate_ssl_conf is not None:
+                driver_args['trust'] = neo4j.TRUST_SYSTEM_CA_SIGNED_CERTIFICATES if validate_ssl_conf \
+                    else neo4j.TRUST_ALL_CERTIFICATES
+            if encrypted_conf is not None:
+                driver_args['encrypted'] = encrypted_conf
+
+            self.driver = GraphDatabase.driver(**driver_args)
         self._extract_iter: Union[None, Iterator] = None
 
         model_class = conf.get(Neo4jExtractor.MODEL_CLASS_CONFIG_KEY, None)
@@ -62,20 +97,6 @@ class Neo4jExtractor(Extractor):
         except Exception as e:
             LOGGER.error("Exception encountered while closing the graph driver", e)
 
-    def _get_driver(self) -> Any:
-        """
-        Create a Neo4j connection to Database
-        """
-        trust = neo4j.TRUST_SYSTEM_CA_SIGNED_CERTIFICATES if self.conf.get_bool(Neo4jExtractor.NEO4J_VALIDATE_SSL) \
-            else neo4j.TRUST_ALL_CERTIFICATES
-        return GraphDatabase.driver(uri=self.graph_url,
-                                    max_connection_lifetime=self.conf.get_int(
-                                        Neo4jExtractor.NEO4J_MAX_CONN_LIFE_TIME_SEC),
-                                    auth=(self.conf.get_string(Neo4jExtractor.NEO4J_AUTH_USER),
-                                          self.conf.get_string(Neo4jExtractor.NEO4J_AUTH_PW)),
-                                    encrypted=self.conf.get_bool(Neo4jExtractor.NEO4J_ENCRYPTED),
-                                    trust=trust)
-
     def _execute_query(self, tx: Any) -> Any:
         """
         Create an iterator to execute sql.
@@ -88,7 +109,9 @@ class Neo4jExtractor(Extractor):
         """
         Execute {cypher_query} and yield result one at a time
         """
-        with self.driver.session() as session:
+        with self.driver.session(
+            database=self.db_name
+        ) as session:
             if not hasattr(self, 'results'):
                 self.results = session.read_transaction(self._execute_query)
 
