@@ -10,6 +10,9 @@ from typing import (
 
 import neo4j
 from neo4j import GraphDatabase
+from neo4j.api import (
+    SECURITY_TYPE_SECURE, SECURITY_TYPE_SELF_SIGNED_CERTIFICATE, parse_neo4j_uri,
+)
 from pyhocon import ConfigFactory, ConfigTree
 
 from databuilder import Scoped
@@ -21,11 +24,13 @@ NEO4J_END_POINT_KEY = 'neo4j_endpoint'
 NEO4J_MAX_CONN_LIFE_TIME_SEC = 'neo4j_max_conn_life_time_sec'
 NEO4J_USER = 'neo4j_user'
 NEO4J_PASSWORD = 'neo4j_password'
+# in Neo4j (v4.0+), we can create and use more than one active database at the same time
+NEO4J_DATABASE_NAME = 'neo4j_database'
+NEO4J_DRIVER = 'neo4j_driver'
 NEO4J_ENCRYPTED = 'neo4j_encrypted'
 """NEO4J_ENCRYPTED is a boolean indicating whether to use SSL/TLS when connecting."""
 NEO4J_VALIDATE_SSL = 'neo4j_validate_ssl'
 """NEO4J_VALIDATE_SSL is a boolean indicating whether to validate the server's SSL/TLS cert against system CAs."""
-
 TARGET_NODES = "target_nodes"
 TARGET_RELATIONS = "target_relations"
 BATCH_SIZE = "batch_size"
@@ -41,8 +46,7 @@ RETAIN_DATA_WITH_NO_PUBLISHER_METADATA = "retain_data_with_no_publisher_metadata
 
 DEFAULT_CONFIG = ConfigFactory.from_dict({BATCH_SIZE: 100,
                                           NEO4J_MAX_CONN_LIFE_TIME_SEC: 50,
-                                          NEO4J_ENCRYPTED: True,
-                                          NEO4J_VALIDATE_SSL: False,
+                                          NEO4J_DATABASE_NAME: neo4j.DEFAULT_DATABASE,
                                           STALENESS_MAX_PCT: 5,
                                           TARGET_NODES: [],
                                           TARGET_RELATIONS: [],
@@ -127,14 +131,36 @@ class Neo4jStalenessRemovalTask(Task):
         else:
             self.marker = conf.get_string(JOB_PUBLISH_TAG)
 
-        trust = neo4j.TRUST_SYSTEM_CA_SIGNED_CERTIFICATES if conf.get_bool(NEO4J_VALIDATE_SSL) \
-            else neo4j.TRUST_ALL_CERTIFICATES
-        self._driver = \
-            GraphDatabase.driver(uri=conf.get_string(NEO4J_END_POINT_KEY),
-                                 max_connection_lifetime=conf.get_int(NEO4J_MAX_CONN_LIFE_TIME_SEC),
-                                 auth=(conf.get_string(NEO4J_USER), conf.get_string(NEO4J_PASSWORD)),
-                                 encrypted=conf.get_bool(NEO4J_ENCRYPTED),
-                                 trust=trust)
+        driver = conf.get(NEO4J_DRIVER, None)
+        if driver:
+            self._driver = driver
+        else:
+            uri = conf.get_string(NEO4J_END_POINT_KEY)
+            driver_args = {
+                'uri': uri,
+                'max_connection_lifetime': conf.get_int(NEO4J_MAX_CONN_LIFE_TIME_SEC),
+                'auth': (conf.get_string(NEO4J_USER), conf.get_string(NEO4J_PASSWORD)),
+            }
+
+            # if URI scheme not secure set `trust`` and `encrypted` to default values
+            # https://neo4j.com/docs/api/python-driver/current/api.html#uri
+            _, security_type, _ = parse_neo4j_uri(uri=uri)
+            if security_type not in [SECURITY_TYPE_SELF_SIGNED_CERTIFICATE, SECURITY_TYPE_SECURE]:
+                default_security_conf = {'trust': neo4j.TRUST_ALL_CERTIFICATES, 'encrypted': True}
+                driver_args.update(default_security_conf)
+
+            # if NEO4J_VALIDATE_SSL or NEO4J_ENCRYPTED are set in config pass them to the driver
+            validate_ssl_conf = conf.get(NEO4J_VALIDATE_SSL, None)
+            encrypted_conf = conf.get(NEO4J_ENCRYPTED, None)
+            if validate_ssl_conf is not None:
+                driver_args['trust'] = neo4j.TRUST_SYSTEM_CA_SIGNED_CERTIFICATES if validate_ssl_conf \
+                    else neo4j.TRUST_ALL_CERTIFICATES
+            if encrypted_conf is not None:
+                driver_args['encrypted'] = encrypted_conf
+
+            self._driver = GraphDatabase.driver(**driver_args)
+
+        self.db_name = conf.get(NEO4J_DATABASE_NAME)
 
     def run(self) -> None:
         """
@@ -304,7 +330,7 @@ class Neo4jStalenessRemovalTask(Task):
 
         start = time.time()
         try:
-            with self._driver.session() as session:
+            with self._driver.session(database=self.db_name) as session:
                 result = session.run(statement, **param_dict)
                 return [record for record in result]
 
